const winston = require('winston');
const serializeError = require('serialize-error');

const config = require('./config/default');
const { buyanMapping } = require('./helpers/mappings');
const {
  formatLevel,
  addVersionInfo,
  formatTimestamp
} = require('./helpers/formaters');

const buildLog = (event, data = null, meta = null, additionalInfos = {}) => {
  if (data instanceof Error) {
    data = {
      error: serializeError(data)
    };
  }

  return {
    ...additionalInfos,
    event,
    data,
    meta
  };
};

const methods = (loggerRef, additionalInfos) => {
  const logMethods = {};
  ['debug', 'info', 'warn', 'error', 'fatal'].forEach(logLevel => {
    logMethods[logLevel] = (event, data, meta) => {
      loggerRef[buyanMapping[logLevel]](
        buildLog(event, data, meta, additionalInfos)
      );
    };
  });

  return logMethods;
};

const create = additionalInfos => {
  const mainLogger = winston.createLogger({
    // Here verbose means info in buyan
    level: buyanMapping[config.logLevel] || buyanMapping['info'],
    format: winston.format.combine(
      formatLevel(),
      addVersionInfo(),
      winston.format.timestamp(),
      formatTimestamp(),
      winston.format.json()
    )
  });

  if (process.env.NODE_ENV === 'production') {
    mainLogger.add(new winston.transports.Http({
      host: 'agents.kosm.io',
      port: 5300
    }));
  }
  mainLogger.add(new winston.transports.Console({}));

  return methods(mainLogger, additionalInfos);
};

module.exports = { create };
